@@ -1,7 +1,10 @@
 # pylint: disable = too-many-lines
 
+from __future__ import annotations
+
 import enum
-from typing import Callable, List, Mapping, Optional, Sequence, Tuple
+from collections.abc import Callable, Sequence
+from typing import Optional
 
 from rflx import expression as expr, model
 from rflx.ada import (
@@ -196,7 +199,7 @@ def substitution_facts(
     embedded: bool = False,
     public: bool = False,
     target_type: Optional[ID] = const.TYPES_BASE_INT,
-) -> Mapping[expr.Name, expr.Expr]:
+) -> dict[expr.Name, expr.Expr]:
     def prefixed(name: str) -> expr.Expr:
         return expr.Variable(ID("Ctx") * name) if not embedded else expr.Variable(name)
 
@@ -305,6 +308,17 @@ def substitution_facts(
 def message_structure_invariant(
     message: model.Message, prefix: str, link: model.Link = None, embedded: bool = False
 ) -> Expr:
+    """
+    Create the invariant that defines a valid message structure.
+
+    This invariant ensures the properties of message fields that can depend on the concrete message
+    path: the field size, the location of the field and the predecessor of the field. This is
+    realized by nested if-expressions. Each if-expression represents a link in the message and
+    checks the validity of the message field and the link condition. The if-expressions form a
+    tree-like structure with the link to the first field as the root and the links to the final
+    field as the leafs.
+    """
+
     def prefixed(name: str) -> expr.Expr:
         return expr.Selected(expr.Variable("Ctx"), name) if not embedded else expr.Variable(name)
 
@@ -415,6 +429,16 @@ def context_predicate(
     message: model.Message, composite_fields: Sequence[model.Field], prefix: str
 ) -> Expr:
     def cursors_invariant() -> Expr:
+        """
+        Create the invariant that defines valid representations of fields.
+
+        Each field cursor represents the state of one parsed or serialized message field.
+        This invariant ensures for all structurally valid fields that
+
+            - the field bounds are inside the range of verified buffer part,
+            - the field size is greater or equal to zero,
+            - and the field value fulfills all constraints of its field type.
+        """
         return ForAllIn(
             "F",
             Variable("Field"),
@@ -461,6 +485,18 @@ def context_predicate(
         )
 
     def valid_predecessors_invariant() -> Expr:
+        """
+        Create the invariant that defines the state of predecessors of valid fields.
+
+        This invariant ensures for all structurally valid message fields that
+
+            - one of its predecessor fields is structurally valid,
+            - the predecessor component in the cursor refers to a valid predecessor,
+            - and the condition on the link between the field and its predecessor is fulfilled.
+
+        This ensures that there is a valid message path from each structurally valid field to the
+        initial field.
+        """
         return AndThen(
             *[
                 If(
@@ -517,6 +553,12 @@ def context_predicate(
         )
 
     def invalid_successors_invariant() -> Expr:
+        """
+        Create the invariant that defines the state of successors of invalid fields.
+
+        This invariant ensures for all invalid message fields that all its successor fields are also
+        invalid.
+        """
         return AndThen(
             *[
                 If(
@@ -582,6 +624,17 @@ def context_predicate(
 
 
 def public_context_predicate() -> Expr:
+    """
+    Create the predicate that defines the relations between the public discriminants of the context.
+
+    This invariant ensures that
+
+        - the positions of the first and last usable bit are inside the range of the buffer,
+        - the size of the usable buffer is greater or equal to zero,
+        - the value of `Last` and `Buffer_Last` is less than the last possible value of their
+          respective type to allow the representation of a size of zero in these edge cases
+        - and the positions of the first and last usable bit are byte aligned.
+    """
     return And(
         GreaterEqual(Call(const.TYPES_TO_INDEX, [Variable("First")]), Variable("Buffer_First")),
         LessEqual(Call(const.TYPES_TO_INDEX, [Variable("Last")]), Variable("Buffer_Last")),
@@ -593,7 +646,7 @@ def public_context_predicate() -> Expr:
     )
 
 
-def context_invariant(message: model.Message, loop_entry: bool = False) -> Sequence[Expr]:
+def context_invariant(message: model.Message, loop_entry: bool = False) -> list[Expr]:
     return [
         Equal(e, LoopEntry(e) if loop_entry else Old(e))
         for e in [
@@ -608,7 +661,7 @@ def context_invariant(message: model.Message, loop_entry: bool = False) -> Seque
 
 def valid_path_to_next_field_condition(
     message: model.Message, field: model.Field, prefix: str
-) -> Sequence[Expr]:
+) -> list[Expr]:
     return [
         If(
             [
@@ -641,7 +694,7 @@ def valid_path_to_next_field_condition(
 
 def context_cursor_unchanged(
     message: model.Message, field: model.Field, predecessors: bool
-) -> List[Expr]:
+) -> list[Expr]:
     lower: model.Field
     upper: model.Field
     if predecessors:
@@ -700,7 +753,7 @@ def sufficient_space_for_field_condition(
 
 def initialize_field_statements(
     field: model.Field, reset_written_last: bool = False
-) -> Sequence[Statement]:
+) -> list[Statement]:
     return [
         CallStatement(
             "Reset_Dependent_Fields",
@@ -773,7 +826,7 @@ def initialize_field_statements(
     ]
 
 
-def field_bit_location_declarations(field_name: Name) -> Sequence[Declaration]:
+def field_bit_location_declarations(field_name: Name) -> list[Declaration]:
     return [
         ObjectDeclaration(
             ["First"],
@@ -904,11 +957,11 @@ def create_sequence_instantiation(
     sequence_type: model.Sequence,
     prefix: str = "",
     flat: bool = False,
-) -> Tuple[List[ContextItem], GenericPackageInstantiation]:
+) -> tuple[list[ContextItem], GenericPackageInstantiation]:
     element_type = sequence_type.element_type
     element_type_package = element_type.package.name
 
-    sequence_context: List[ContextItem] = []
+    sequence_context: list[ContextItem] = []
     sequence_package: GenericPackageInstantiation
     if isinstance(element_type, model.Message):
         element_type_identifier = ID(
@@ -1042,14 +1095,14 @@ def conditional_field_size(field: model.Field, message: model.Message, prefix: s
     )
 
 
-def message_parameters(message: model.Message) -> List[Parameter]:
+def message_parameters(message: model.Message) -> list[Parameter]:
     return [
         Parameter([p.name], ada_type_identifier(t.identifier))
         for p, t in message.parameter_types.items()
     ]
 
 
-def initialize_conditions(message: model.Message) -> Sequence[Expr]:
+def initialize_conditions(message: model.Message) -> list[Expr]:
     return [
         *[
             Equal(
