@@ -1,7 +1,9 @@
+from __future__ import annotations
+
 import logging
 import re
+from collections.abc import Sequence
 from pathlib import Path
-from typing import Sequence
 
 from pydotplus import Dot, Edge, Node
 
